/*
 *  ADS – A Java package of algorithms and data structures for educational purposes.  This software was
 *  originally created for the book Winder & Roberts, "Developing Java Software", Wiley.
 *
 *  Copyright © 1997–2008,2010 Russel Winder
 *
 *  This file is part of ADS.
 *
 *  ADS is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General
 *  Public License as published by the Free Software Foundation, either version 3 of the License, or (at
 *  your option) any later version.
 *
 *  ADS is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
 *  implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General
 *  Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License along with ADS.  If not, see
 *  <http://www.gnu.org/licenses/>.
 */
package org.devjavasoft.ads ;
/**
 *  An abstract <code>Dequeue</code> to act as a superclass for all the concrete sequences.  Provides all
 *  the default implementations that are representation independent.
 *
 *  @author Russel Winder
 */
public abstract class AbstractDequeue<T> extends AbstractQueue<T> implements Dequeue<T> {
  @Override public abstract void headAdd ( T o ) ;
  @Override public abstract void tailAdd ( T o ) ;
  @Override public abstract T headRemove ( ) ;
  @Override public abstract T tailRemove ( ) ;
  @Override public abstract T headPeek ( ) ;
  @Override public abstract T tailPeek ( ) ;
  @Override public T peek ( ) { return headPeek ( ) ; }
  @Override public T remove ( ) { return headRemove ( ) ; }
  @Override public void add ( final T o ) { tailAdd ( o ) ; }
  @Override public boolean equals ( final Object o ) {
    if ( o == this ) { return true ; }
    if ( ( o == null ) || ! ( o instanceof Dequeue<?> ) ) { return false ; }
    return super.equals(o) ;
  }
  @Override public Dequeue<T> clone ( ) { return (Dequeue<T>) super.clone ( ) ; }
}
