/*
 *  ADS – A Java package of algorithms and data structures for educational purposes.  This software was
 *  originally created for the book Winder & Roberts, "Developing Java Software", Wiley.
 *
 *  Copyright © 1997–2008,2010 Russel Winder
 *
 *  This file is part of ADS.
 *
 *  ADS is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General
 *  Public License as published by the Free Software Foundation, either version 3 of the License, or (at
 *  your option) any later version.
 *
 *  ADS is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
 *  implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General
 *  Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License along with ADS.  If not, see
 *  <http://www.gnu.org/licenses/>.
 */
package org.devjavasoft.ads ;
/**
 *  An abstract <code>Relation</code>.
 *
 *  @author Russel Winder
 */
public abstract class AbstractRelation<D, R> extends AbstractSet<Pair<D, R>> implements Relation<D, R> {
  @Override public void add ( final D d , final R r ) { add ( new Pair<D, R> ( d , r ) ) ; }
  @Override public boolean remove ( final D d , final R r ) { return remove ( new Pair<D, R> ( d , r ) ) ; }
  @Override public abstract  InputIterator<D> domain ( ) ;
  @Override public abstract  InputIterator<D> domain ( R r ) ;
  @Override public abstract  InputIterator<R> range ( ) ;
  @Override public abstract  InputIterator<R> range ( D d ) ;
  @SuppressWarnings ( "unchecked" )
  @Override public AbstractRelation<D, R> clone ( ) { return (AbstractRelation<D, R>) super.clone ( ) ; }
  @Override public boolean equals ( final Object o ) {
    if ( o == this ) { return true ; }
    if ( ( o == null ) || ! ( o instanceof Relation<?,?> ) ) { return false ; }
    return super.equals ( o ) ;
  }
}
