/*
 *  ADS – A Java package of algorithms and data structures for educational purposes.  This software was
 *  originally created for the book Winder & Roberts, "Developing Java Software", Wiley.
 *
 *  Copyright © 1997–2008,2010 Russel Winder
 *
 *  This file is part of ADS.
 *
 *  ADS is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General
 *  Public License as published by the Free Software Foundation, either version 3 of the License, or (at
 *  your option) any later version.
 *
 *  ADS is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
 *  implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General
 *  Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License along with ADS.  If not, see
 *  <http://www.gnu.org/licenses/>.
 */
package org.devjavasoft.ads ;
/**
 *  An abstract <code>Set</code>.
 *
 *  @author Russel Winder
 */
public abstract class AbstractSet<T> extends AbstractContainer<T> implements Set<T> {
  @Override public Set<T> union ( final Set<T> s ) {
    final Set<T> newSet = clone ( ) ;
    for ( InputIterator<T> i = s.iterator ( ) ; ! i.atEnd ( ) ; i.advance ( ) ) { newSet.add ( i.get ( ) ) ; }
    return newSet ;
  }
  @Override public Set<T> intersection ( final Set<T> s ) {
    final Set<T> newSet = clone ( ) ;
    for ( InputIterator<T> i = newSet.iterator ( ) ; ! i.atEnd ( ) ; i.advance ( ) ) {
      final T o = i.get ( ) ;
      if ( ! s.contains ( o ) ) { newSet.remove ( o ) ; }
    }
    return newSet ;
  }
  @Override public Set<T> difference ( final Set<T> s ) {
    final Set<T> newSet = clone ( ) ;
    for ( InputIterator<T> i = s.iterator ( ) ; ! i.atEnd ( ) ; i.advance ( ) ) {
      newSet.remove ( i.get ( ) ) ;
    }
    return newSet ;
  }
  @SuppressWarnings ( "unchecked" )
  @Override public boolean equals ( final Object o ) {
    if ( o == this ) { return true ; }
    if ( ( o == null ) || ! ( o instanceof Set ) ) { return false ; }
    final Set<T> other = (Set<T>) o ;
    for ( final T item : this ) {
      if ( ! other.contains ( item ) ) { return false ; }
    }
    for ( final T item : other ) {
      if ( ! contains ( item ) ) { return false ; }
    }
    return true ;
  }
  @Override public AbstractSet<T> clone ( ) { return (AbstractSet<T>) super.clone ( ) ; }
}
