/*
 *  ADS – A Java package of algorithms and data structures for educational purposes.  This software was
 *  originally created for the book Winder & Roberts, "Developing Java Software", Wiley.
 *
 *  Copyright © 1997–2008,2010 Russel Winder
 *
 *  This file is part of ADS.
 *
 *  ADS is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General
 *  Public License as published by the Free Software Foundation, either version 3 of the License, or (at
 *  your option) any later version.
 *
 *  ADS is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
 *  implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General
 *  Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License along with ADS.  If not, see
 *  <http://www.gnu.org/licenses/>.
 */
package org.devjavasoft.ads ;
/**
 *  An abstract <code>Stack</code> to act as a superclass for all the concrete implementations.  Provides
 *  all the default implementations that are representation independent.
 *
 *  @author Russel Winder
 */
public abstract class AbstractStack<T> extends AbstractContainer<T> implements Stack<T> {
  @Override public abstract T remove ( ) ;
  @Override public void push ( final T o ) { add ( o ) ; }
  @Override public T pop ( ) { return remove ( ) ; }
  @Override public abstract T peek ( ) ;
  @Override public T top ( ) { return peek ( ) ; }
  @SuppressWarnings ( "unchecked" )
  @Override public boolean equals ( final Object o ) {
    if ( this == o ) { return true ; }
    if ( ( o == null ) || ! ( o instanceof Stack ) || ( size ( ) != ( (Stack<T>) o ).size ( ) ) ) {
      return false ;
    }
    final InputIterator<T> i1 = iterator ( ) ;
    final InputIterator<T> i2 = ( (Stack<T>) o).iterator ( ) ;
    for ( ; ! i1.atEnd ( ) && ! i2.atEnd ( ) ; i1.advance ( ) , i2.advance ( ) ) {
      final T o1 = i1.get ( ) ;
      final T o2 = i2.get ( ) ;
      if ( ( o1 == null ) || ( o2 == null ) || ! o1.equals ( o2 ) ) { return false ; }
    }
    assert i1.atEnd ( ) && i2.atEnd ( )  ;
    return true ;
  }
  @Override public AbstractStack<T> clone ( ) { return (AbstractStack<T>) super.clone ( ) ; }
  @Override public boolean remove ( final T o ) { throw new InvalidOperationException ( ) ; }
}
