/*
 *  ADS – A Java package of algorithms and data structures for educational purposes.  This software was
 *  originally created for the book Winder & Roberts, "Developing Java Software", Wiley.
 *
 *  Copyright © 1997–2008,2010 Russel Winder
 *
 *  This file is part of ADS.
 *
 *  ADS is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General
 *  Public License as published by the Free Software Foundation, either version 3 of the License, or (at
 *  your option) any later version.
 *
 *  ADS is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
 *  implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General
 *  Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License along with ADS.  If not, see
 *  <http://www.gnu.org/licenses/>.
 */
package org.devjavasoft.ads ;
/**
 *  All bidirectional iterators over containers in ADS conform to this interface.  A bidirectional iterator
 *  is a forward iterator where moving backwards as well as forwards in the container makes sense.
 *
 *  @author Russel Winder
 */
public interface BiDirectionalIterator<T> extends ForwardIterator<T> {
  /**
   *  Move back to the previous element in the iteration over the container.
   */
  void retreat ( ) ;
  /**
   *  Move backward to the <code>increment</code>th item before the current one in the iteration over the
   *  container.  If <code>increment</code> is larger than the number of items prior to this in the
   *  container the iterator is positioned at the beginning.
   *
   *  @param increment the number of items to retreat by.
   */
  void retreat ( int increment ) ;
  /**
   *  Deliver a clone of this <code>BiDirectionalIterator</code>.
   *
   *  @return a reference to a clone of this <code>BiDirectionalIterator</code>.
   */
  @Override BiDirectionalIterator<T> clone ( ) ;
}
