/*
 *  ADS – A Java package of algorithms and data structures for educational purposes.  This software was
 *  originally created for the book Winder & Roberts, "Developing Java Software", Wiley.
 *
 *  Copyright © 1997–2008,2010 Russel Winder
 *
 *  This file is part of ADS.
 *
 *  ADS is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General
 *  Public License as published by the Free Software Foundation, either version 3 of the License, or (at
 *  your option) any later version.
 *
 *  ADS is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
 *  implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General
 *  Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License along with ADS.  If not, see
 *  <http://www.gnu.org/licenses/>.
 */
package org.devjavasoft.ads ;
/**
 *  An implementation of a <code>Mapping</code> using a  <code>ChainedHashTable</code>.
 *
 *  @author Russel Winder
 */
public class ChainedHashTableMapping<D, R> extends ChainedHashTableRelation<D, R> implements Mapping<D, R> {
  @Override public void add ( final Pair<D, R> o ) { storage.add ( o ) ; }
  @Override public boolean remove ( final Pair<D, R> o) { return storage.remove ( o ) ; }
  @Override public boolean equals ( final Object o ) {
    if ( this == o ) { return true ; }
    if ( ( o == null ) || ! ( o instanceof ChainedHashTableMapping<?,?> ) ) { return false ; }
    return super.equals ( o ) ;
  }
  @Override public ChainedHashTableMapping<D, R> clone ( ) {
    final ChainedHashTableMapping<D, R> t = (ChainedHashTableMapping<D, R>) super.clone ( ) ;
    if ( ! isEmpty ( ) ) { t.storage = storage.clone ( ) ; }
    return t ;
  }
  @Override public final R getValue ( final D d ) { return storage.getValue ( d ) ; }
}
