/*
 *  ADS – A Java package of algorithms and data structures for educational purposes.  This software was
 *  originally created for the book Winder & Roberts, "Developing Java Software", Wiley.
 *
 *  Copyright © 1997–2008,2010 Russel Winder
 *
 *  This file is part of ADS.
 *
 *  ADS is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General
 *  Public License as published by the Free Software Foundation, either version 3 of the License, or (at
 *  your option) any later version.
 *
 *  ADS is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
 *  implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General
 *  Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License along with ADS.  If not, see
 *  <http://www.gnu.org/licenses/>.
 */
package org.devjavasoft.ads ;
/**
 *  A <code>Pair</code> that conforms to <code>Comparable</code> so that it can be used in situations that
 *  require orders ( either total or partial ).  We require that the first of the pair is of a comparable
 *  type and so provides an order relation on the pairs by having an order relation on the first item of the
 *  pair.
 *
 *  @author Russel Winder
 */
public class ComparablePair<A extends Comparable<A>, B> extends Pair<A, B> implements Comparable<ComparablePair<A, B>> {
  /**
   *  Construct a <code>ComparablePair&lt;A, B></code>.
   *
   *  @param a the first item for test.
   *  @param b the second item for test.
   */
  public ComparablePair ( final A a , final B b ) { super ( a , b ) ; }
  /**
   *  The order relation on these pairs.  There can be no total order on the pair only various partial
   *  orders.  In general, the three obvious orders are by first of the pair, by second of the pair and, if
   *  there is an arithmetic on the elements, by modulus of the pair.  We have only required the first of
   *  the pair to be comparable so the obvious order relation is on first of the pair.  We leave it to
   *  external <code>Comparator</code> implementations to handle the other two.
   */
  @Override public int compareTo ( final ComparablePair<A, B> o ) { return a.compareTo ( o.a ) ; }
  @Override public ComparablePair<A, B> clone ( ) { return (ComparablePair<A, B>) super.clone ( ) ; }
}
