/*
 *  ADS – A Java package of algorithms and data structures for educational purposes.  This software was
 *  originally created for the book Winder & Roberts, "Developing Java Software", Wiley.
 *
 *  Copyright © 1997–2008,2010 Russel Winder
 *
 *  This file is part of ADS.
 *
 *  ADS is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General
 *  Public License as published by the Free Software Foundation, either version 3 of the License, or (at
 *  your option) any later version.
 *
 *  ADS is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
 *  implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General
 *  Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License along with ADS.  If not, see
 *  <http://www.gnu.org/licenses/>.
 */
package org.devjavasoft.ads ;
/**
 *  All forward iterators over containers in ADS conform to this interface.  A forward iterator is an input
 *  iterator that can write to the container.
 *
 *  @author Russel Winder
 */
public interface ForwardIterator<T> extends InputIterator<T> {
  /**
   *  Amend the value at the current position in the container of the iterator.  This operation only really
   *  makes sense for containers with no relation on the contained items.  Iterators working a container
   *  that must preserve a relation may choose to throw an exception or may choose to delete the item and
   *  insert the item in a relation preserving way which will invalidate the iterator.
   *
   *  @param o the value to replace the current value with.
   *  @exception AccessEmptyContainerException if there are no items in the container.
   *  @exception InvalidIteratorException if the iterator is not referring to a valid element.
   */
  void set ( T o ) ;
  /**
   *  Deliver a clone of this <code>ForwardIterator</code>.
   *
   *  @return a reference to a clone of this <code>ForwardIterator</code>.
   */
  @Override ForwardIterator<T> clone ( ) ;
}
