/*
 *  ADS – A Java package of algorithms and data structures for educational purposes.  This software was
 *  originally created for the book Winder & Roberts, "Developing Java Software", Wiley.
 *
 *  Copyright © 1997–2008 Russel Winder
 *
 *  This file is part of ADS.
 *
 *  ADS is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General
 *  Public License as published by the Free Software Foundation, either version 3 of the License, or (at
 *  your option) any later version.
 *
 *  ADS is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
 *  implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General
 *  Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License along with ADS.  If not, see
 *  <http://www.gnu.org/licenses/>.
 */
package org.devjavasoft.ads ;
/**
 *  A class to act as a definer of hash function method objects.  A hash value is a positive integer.
 *
 *  @author Russel Winder
 */
public class HashFunction<T> {
  /**
   *  The hash algorithm encapsulated as a method.
   *
   *  @param o the object to hash.
   *  @return the hash code of the object provided as parameter.
   */
  public int hash ( final T o ) {
    //
    //  It is interesting that in the Java library there is a function in the package java.lang.Math called
    //  abs which ensures that a value is positive and yet it is unused in the types java.util.Hashtable and
    //  com.objectspace.jgl.HashMap.  abs is basically programmed as:
    //
    //      public int abs(int a) {
    //          return a < 0 ? -a : a ;
    //      }
    //
    //  which is deemed to be inefficient and hence not usable: everyone assumes 2's complement 32-bit ints
    //  (which is fair since that is what the JVM defines int to be) and simply masks out the sign bit.
    //  Essentially an efficiency measure since no selection is needed.
    //
    return ( o.hashCode ( ) & 0x7FFFFFFF ) ;
   }
}
