/*
 *  ADS – A Java package of algorithms and data structures for educational purposes.  This software was
 *  originally created for the book Winder & Roberts, "Developing Java Software", Wiley.
 *
 *  Copyright © 1997–2008,2010 Russel Winder
 *
 *  This file is part of ADS.
 *
 *  ADS is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General
 *  Public License as published by the Free Software Foundation, either version 3 of the License, or (at
 *  your option) any later version.
 *
 *  ADS is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
 *  implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General
 *  Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License along with ADS.  If not, see
 *  <http://www.gnu.org/licenses/>.
 */
package org.devjavasoft.ads ;
/**
 *  A comparator to represent the not equal to order relation.
 *
 *  <p>This comparator could require the parameter type <code>T</code> to implement <code>Comparable</code>
 *  and then use the <code>compareTo</code> operation to discover equality or not but there is an
 *  alternative.  Assuming that <code>T.equals</code> is defined as value equality and not identity equality
 *  (which is likely the case for all types that conform to <code>Comparable</code>) the better alternative
 *  is to use the <code>equals</code> method.  Not constraining the type parameter gives this comparator
 *  wider applicability.</p>
 *
 *  @author Russel Winder
 */
public class NotEqualToComparator<T extends Comparable<T>> implements Comparator<T> {
  @Override public final boolean relation ( final T a , final T b ) { return ! a.equals ( b ) ; }
  @Override public final boolean equals ( final Object o ) {
    if ( o == this ) { return true ; }
    if ( ( o == null ) || ! ( o instanceof NotEqualToComparator<?> ) ) { return false ; }
    //  Since we only have access to the raw type and not the full type we cannot distinguish different
    //  types of EqualToComparator so there will be apparently false positives.
    return true ;
  }
}
