/*
 *  ADS – A Java package of algorithms and data structures for educational purposes.  This software was
 *  originally created for the book Winder & Roberts, "Developing Java Software", Wiley.
 *
 *  Copyright © 1997–2008,2010 Russel Winder
 *
 *  This file is part of ADS.
 *
 *  ADS is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General
 *  Public License as published by the Free Software Foundation, either version 3 of the License, or (at
 *  your option) any later version.
 *
 *  ADS is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
 *  implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General
 *  Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License along with ADS.  If not, see
 *  <http://www.gnu.org/licenses/>.
 */
package org.devjavasoft.ads ;
/**
 *  A bag implemented using an <code>OpenHashTable</code>.
 *
 *  @author Russel Winder
 */
public class OpenHashTableBag<T> extends OpenHashTableSet<T> implements Bag<T> {
  /**
   *  Construct an <code>OpenHashTableBag</code>.
   */
  public OpenHashTableBag ( ) { storage = new OpenHashTable<T, T> ( ) ; }
  @Override public OpenHashTableBag<T> clone() {
    final OpenHashTableBag<T> b = (OpenHashTableBag<T>) super.clone ( ) ;
    b.storage = storage.clone ( ) ;
    return b ;
  }
  @Override public int countOf ( final T o ) {
    int count = 0 ;
    for ( InputIterator<Pair<T, T>> i = storage.iterator ( ) ; ! i.atEnd ( ) ; i.advance ( ) ) {
      if ( i.get ( ).getA ( ).equals ( o ) ) { ++count ; }
    }
    return count ;
  }
}
