/*
 *  ADS – A Java package of algorithms and data structures for educational purposes.  This software was
 *  originally created for the book Winder & Roberts, "Developing Java Software", Wiley.
 *
 *  Copyright © 1997–2008,2010 Russel Winder
 *
 *  This file is part of ADS.
 *
 *  ADS is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General
 *  Public License as published by the Free Software Foundation, either version 3 of the License, or (at
 *  your option) any later version.
 *
 *  ADS is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
 *  implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General
 *  Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License along with ADS.  If not, see
 *  <http://www.gnu.org/licenses/>.
 */
package org.devjavasoft.ads ;
/**
 *  A bag implemented using an <code>OrderedBinaryTree</code>.
 *
 *  @author Russel Winder
 */
public class OrderedBinaryTreeBag<T> extends OrderedBinaryTreeSet<T> implements Bag<T> {
  /**
   *  Construct an <code>OrderedBinaryTreeBag</code> using the equality relation
   *  <code>EqualToComparator</code> as a method object encapsulating the <code>equals</code> method.  The
   *  less than order relation must be provided by the instantiator.
   *
   *  @param c the comparator to use for providing an order.
   */
  //  TODO :  Really ought to make the default here a RedBlackTree.
  public OrderedBinaryTreeBag ( final Comparator<? super T> c ) {
    //  The superclass uses an ordered binary tree without duplicates all we need to do is to replace the
    //  representation to ensure that duplicates are allowed.
    super ( c ) ;
    storage = new OrderedBinaryTree<T> ( c , new EqualToComparator<T> ( ) , true ) ;
  }
  /**
   *  Construct an <code>OrderedBinaryTreeBag</code> using the equality relation
   *  (<code>EqualToComparator</code>) as a method object encapsulating the <code>equals</code> method and
   *  copying in the values from the <code>Container</code>.
   *
   *  @param cp  the comparator to use for providing an order.
   *  @param ct  a container containing items that should be added to this tree.  Shallow copy.
   */
  public OrderedBinaryTreeBag ( final Comparator<? super T> cp , final Container<T> ct ) {
    this ( cp ) ;
    copyIn ( ct ) ;
  }
  @Override public OrderedBinaryTreeBag<T> clone ( ) {
    final OrderedBinaryTreeBag<T> b = (OrderedBinaryTreeBag<T>) super.clone ( ) ;
    b.storage = storage.clone ( ) ;
    return b ;
  }
  @Override public int countOf ( final T o ) {
    // This implements an O(log(n)) linear search.
    int count = 0 ;
    for ( OrderedBinaryTree<T>.SearchIterator si = storage.searchFor ( o ) ; ! si.atEnd ( ) ; si.advance ( ) ) {
      ++count ;
    }
    return count ;
  }
}
