/*
 *  ADS – A Java package of algorithms and data structures for educational purposes.  This software was
 *  originally created for the book Winder & Roberts, "Developing Java Software", Wiley.
 *
 *  Copyright © 1997–2008 Russel Winder
 *
 *  This file is part of ADS.
 *
 *  ADS is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General
 *  Public License as published by the Free Software Foundation, either version 3 of the License, or (at
 *  your option) any later version.
 *
 *  ADS is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
 *  implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General
 *  Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License along with ADS.  If not, see
 *  <http://www.gnu.org/licenses/>.
 */
package org.devjavasoft.ads ;
/**
 *  A class representing a pair (a, b) where a is of type <code>A</code> and b is of type <code>B</code>.
 *  <code>Pair</code>s are immutable.
 *
 *  @author Russel Winder
 */
public class Pair<A, B> implements Cloneable {
  /**
   *  The first of the <code>Pair</code>
   */
  protected A a ;
  /**
   *  The second of the <code>Pair</code>.
   */
  protected B b ;
  /**
   *  Construct a <code>Pair</code>.
   *
   *  @param a the first item.
   *  @param b the second item.
   */
  public Pair ( final A a , final B b ) {
    this.a = a ;
    this.b = b ;
  }
  /**
   *  Accessor for a.
   */
  public final A getA ( ) { return a ; }
  /**
   *  Accessor for b.
   */
  public final B getB ( ) { return b ; }
  @SuppressWarnings ( "unchecked" )
  @Override public boolean equals ( final Object o ) {
    if ( this == o ) { return true ; }
    if ( (o == null ) || ! ( o instanceof Pair ) ) { return false ; }
    final A oa = ( (Pair<A, B>) o ).getA ( ) ;
    final B ob = ( (Pair<A, B>) o ).getB ( ) ;
    if ( (( a == null ) && ( oa != null ) ) || ( ( oa == null ) && ( a != null ) ) ||
         ( ( b == null ) && ( ob != null ) ) || ( ( ob == null ) && ( b != null ) ) ||
         ( ( a != null ) && ! a.equals ( oa ) ) || ( ( b != null ) && ! b.equals ( ob ) ) ) {
      return false ;
    }
    return true ;
  }
  @SuppressWarnings ( "unchecked" )
  @Override public Pair<A, B> clone ( ) {
    try { return (Pair<A, B>) super.clone ( ) ; }
    catch ( CloneNotSupportedException e ) { throw new InternalError ( )  ; }
  }
  @Override public String toString ( ) { return "(" + a + ", " + b + ')' ; }
}
