/*
 *  ADS – A Java package of algorithms and data structures for educational purposes.  This software was
 *  originally created for the book Winder & Roberts, "Developing Java Software", Wiley.
 *
 *  Copyright © 1997–2008,2010 Russel Winder
 *
 *  This file is part of ADS.
 *
 *  ADS is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General
 *  Public License as published by the Free Software Foundation, either version 3 of the License, or (at
 *  your option) any later version.
 *
 *  ADS is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
 *  implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General
 *  Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License along with ADS.  If not, see
 *  <http://www.gnu.org/licenses/>.
 */
package org.devjavasoft.ads ;
/**
 *  A comparator encapsulating the ability to apply a <code>Comparator</code> to the second element of a
 *  <code>Pair</code>.
 *
 *  @author Russel Winder
 */
public class PairBComparator<A , B , T extends Pair<A , B>> implements Comparator<T> {
  private final Comparator<B> comparator ;
  public PairBComparator ( final Comparator<B> c ) { comparator = c ; }
  @Override public final boolean relation ( final T a , final T b ) { return comparator.relation ( a.getB ( ) , b.getB ( ) ) ; }
  @SuppressWarnings ( "unchecked" )
  @Override public final boolean equals ( final Object o ) {
    if ( o == this ) { return true ; }
    if ( ( o == null ) || ! ( o instanceof PairBComparator ) ) { return false ; }
    //  Since we only have access to the raw type and not the full type we cannot distinguish different
    //  types of EqualToComparator so there will be apparently false positives.
    return comparator.equals ( ( (PairBComparator<A,B,T>) o ).comparator ) ;
  }
}
