/*
 *  ADS – A Java package of algorithms and data structures for educational purposes.  This software was
 *  originally created for the book Winder & Roberts, "Developing Java Software", Wiley.
 *
 *  Copyright © 1997–2008,2010 Russel Winder
 *
 *  This file is part of ADS.
 *
 *  ADS is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General
 *  Public License as published by the Free Software Foundation, either version 3 of the License, or (at
 *  your option) any later version.
 *
 *  ADS is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
 *  implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General
 *  Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License along with ADS.  If not, see
 *  <http://www.gnu.org/licenses/>.
 */
package org.devjavasoft.ads ;
/**
 *  All random access iterators over containers in ADS conform to this interface.  A random access iterator
 *  is a bidirectional iterator that can return the index in the container of the item pointed to.
 *
 *  @author Russel Winder
 */
public interface RandomAccessIterator<T> extends BiDirectionalIterator<T> {
  /**
   *  Return the index in the container (which must effectively be a sequence) of the item the iterator
   *  points at.
   *
   *  @return the index in the container of the item the iterator points at.
   */
  int index ( ) ;
  /**
   *  Deliver a clone of this <code>RandomAccessIterator</code>.
   *
   *  @return a reference to a clone of this <code>RandomAccessIterator</code>.
   */
  @Override RandomAccessIterator<T> clone ( ) ;
}
