/*
 *  ADS – A Java package of algorithms and data structures for educational purposes.  This software was
 *  originally created for the book Winder & Roberts, "Developing Java Software", Wiley.
 *
 *  Copyright © 1997–2008 Russel Winder
 *
 *  This file is part of ADS.
 *
 *  ADS is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General
 *  Public License as published by the Free Software Foundation, either version 3 of the License, or (at
 *  your option) any later version.
 *
 *  ADS is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
 *  implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General
 *  Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License along with ADS.  If not, see
 *  <http://www.gnu.org/licenses/>.
 */
package org.devjavasoft.ads.sorting ;
import org.devjavasoft.ads.Comparator ;
import org.devjavasoft.ads.Heap ;
import org.devjavasoft.ads.Sequence ;
/**
 *  Sort a sequence of <code>T</code>s using Heap Sort.  This is guaranteed to be an O(n.log(n)) sort.
 *
 *  @see Comparator
 *  @see Heap
 *  @author Russel Winder
 */
public class HeapSort {
  private HeapSort ( ) { }
  /**
   *  The sort operation for primitive arrays.
   *
   *  @param a the primitive array to be sorted.
   *  @param c the <code>Comparator</code> used to compare the <code>T</code> during the sort process.
   */
  public static <T> void sort ( final T[] a , final Comparator<? super T> c ) {
    final Heap<T> heap = new Heap<T> ( a.length , c ) ;
    for ( final T anA : a ) { heap.add ( anA ) ; }
    for ( int i = 0 ; i < a.length ; ++i ) { a[i] = heap.remove ( ) ;  }
  }
  /**
   *  The sort operation for sequences.
   *
   *  @param s the <code>Sequence</code> to be sorted.
   *  @param c the <code>Comparator&lt;? super T></code> used to compare the <code>T</code>s during the sort
   *  process.
   */
  public static <T> void sort ( final Sequence<T> s , final Comparator<? super T> c ) {
    final Heap<T> heap = new Heap<T> ( s , c ) ;
    s.makeEmpty ( ) ;
    while ( ! heap.isEmpty ( ) ) { s.add ( heap.remove ( ) ) ; }
  }
}
