/*
 *  ADS – A Java package of algorithms and data structures for educational purposes.  This software was
 *  originally created for the book Winder & Roberts, "Developing Java Software", Wiley.
 *
 *  Copyright © 1997–2008 Russel Winder
 *
 *  This file is part of ADS.
 *
 *  ADS is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General
 *  Public License as published by the Free Software Foundation, either version 3 of the License, or (at
 *  your option) any later version.
 *
 *  ADS is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
 *  implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General
 *  Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License along with ADS.  If not, see
 *  <http://www.gnu.org/licenses/>.
 */
package org.devjavasoft.ads.sorting ;
import org.devjavasoft.ads.Comparator ;
/**
 *  Sort a primitive array of <code>T</code>s using Insertion Sort.  This is an O(n^2) sort.
 *
 *  @author Russel Winder
 */
public class InsertionSortBasic {
  private InsertionSortBasic ( ) { }
  /**
   *  The sort operation.
   *
   *  @param a the primitive array to be sorted.
   *  @param c the <code>Comparator</code> used to compare the <code>T</code>s during the sort process.
   */
  public static <T> void sort ( final T[] a , final Comparator<? super T> c ) {
    for ( int i = 1 ; i < a.length ; ++i ) {
      //  Work from the current location back through the sorted part of the array moving the value from the
      //  current location into the right place in the sorted part of the array.
      for ( int j = i - 1 ; ( j >= 0 ) && c.relation ( a[j+1] , a[j] ) ; --j ) {
        final T temp = a[j] ;
        a[j] = a[j+1] ;
        a[j+1] = temp ;
      }
    }
  }
}
