/*
 *  ADS – A Java package of algorithms and data structures for educational purposes.  This software was
 *  originally created for the book Winder & Roberts, "Developing Java Software", Wiley.
 *
 *  Copyright © 1997–2008,2010 Russel Winder
 *
 *  This file is part of ADS.
 *
 *  ADS is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General
 *  Public License as published by the Free Software Foundation, either version 3 of the License, or (at
 *  your option) any later version.
 *
 *  ADS is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
 *  implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General
 *  Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License along with ADS.  If not, see
 *  <http://www.gnu.org/licenses/>.
 */
package org.devjavasoft.ads.sorting ;
import java.io.Closeable ;
import java.io.IOException ;
/**
 *  An interface defining the concept of a record reader, the sort of thing capable of reading a
 *  <code>SortableRecord</code> from a file.
 *
 *  @see SortableRecord
 *  @see SortableRecordWriter
 *  @author Russel Winder
 */
public interface SortableRecordReader extends Closeable {
  /**
   *  Read a <code>SortableRecord</code>.
   *
   *  @return the <code>SortableRecord</code> just read.
   *  @exception IOException on any read error.
   */
  SortableRecord readSortableRecord ( ) throws IOException ;
  /**
   *  Close a <code>SortableRecordReader</code>.
   *
   *  @exception IOException on any close error.
   */
  @Override void close ( ) throws IOException ;
  /**
   *  Mark an input stream.  Needed for polyphase merge sort.
   *
   *  @param lookAheadLimit the distance for lookahead.
   *  @exception IOException on any mark error.
   *  @see java.io.BufferedReader#mark
   */
  void mark ( int lookAheadLimit ) throws IOException ;
  /**
   *  Move back to the mark. Needed for polyphose merge sort.
   *
   *  @exception IOException on any reset error.
   *  @see java.io.BufferedReader#reset
   */
  void reset ( ) throws IOException ;
}
