/*
 *  ADS – A Java package of algorithms and data structures for educational purposes.  This software was
 *  originally created for the book Winder & Roberts, "Developing Java Software", Wiley.
 *
 *  Copyright © 1997–2008,2010 Russel Winder
 *
 *  This file is part of ADS.
 *
 *  ADS is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General
 *  Public License as published by the Free Software Foundation, either version 3 of the License, or (at
 *  your option) any later version.
 *
 *  ADS is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
 *  implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General
 *  Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License along with ADS.  If not, see
 *  <http://www.gnu.org/licenses/>.
 */
package org.devjavasoft.ads.sorting ;
import java.io.Closeable ;
import java.io.IOException ;
/**
 *  An interface defining the concept of a record writer, the sort of thing capable of writing a
 *  <code>SortableRecord</code> to a file.
 *
 *  @see SortableRecord
 *  @see SortableRecordReader
 *  @author Russel Winder
 */
public interface SortableRecordWriter extends Closeable {
  /**
   *  Write a <code>SortableRecord</code>.
   *
   *  @param r the <code>SortableRecord</code> to be written.
   *  @exception IOException on any write error.
   */
  void writeSortableRecord ( SortableRecord r ) throws IOException ;
  /**
   *  Close a <code>SortableRecordWriter</code>.
   *
   *  @exception IOException on any close error.
   */
  @Override void close ( ) throws IOException ;
}
