/*
 *  ADS – A Java package of algorithms and data structures for educational purposes.  This software was
 *  originally created for the book Winder & Roberts, "Developing Java Software", Wiley.
 *
 *  Copyright © 1997–2008 Russel Winder
 *
 *  This file is part of ADS.
 *
 *  ADS is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General
 *  Public License as published by the Free Software Foundation, either version 3 of the License, or (at
 *  your option) any later version.
 *
 *  ADS is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
 *  implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General
 *  Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License along with ADS.  If not, see
 *  <http://www.gnu.org/licenses/>.
 */
package org.devjavasoft.ads.tests ;

import org.devjavasoft.ads.ComparablePair ;

import org.testng.annotations.Test ;
import static org.testng.Assert.assertEquals ;
import static org.testng.Assert.assertFalse ;

/**
 *  A class to test the class <code>ComaprablePair</code>.
 *
 *  @author Russel Winder
 */
public class ComparablePair_Test {
  private final Integer i = 10 ;
  private final String s = "flobadob" ;
  private final ComparablePair<Integer, String> p = new ComparablePair<Integer, String> ( i , s ) ;
  @Test public void comparablePairAccessors ( ) {
    assertEquals ( p.getA ( ) , i ) ;
    assertEquals ( p.getB ( ) , s ) ;
  }
  @Test public void comparablePairClone ( ) {
    final ComparablePair<Integer, String>q = p.clone ( ) ;
    assertEquals ( q.getA ( ) , i ) ;
    assertEquals ( q.getB ( ) , s ) ;
  }
  @Test public void comparablePairEqualToTrueTrivial ( ) {
    assertEquals ( p , p ) ;
  }
  @Test public void comparablePairEqualToTrue ( ) {
    final ComparablePair<Integer, String>q = p.clone ( ) ;
    assertEquals ( p , q ) ;
    assertEquals ( q , p ) ;
  }
  @Test public void comparablePairEqualToTrueNulls ( ) {
    final ComparablePair<Integer, String>q = new ComparablePair<Integer, String> ( null , null ) ;
    final ComparablePair<Integer, String>r = new ComparablePair<Integer, String> ( null , null ) ;
    assertEquals ( q , r ) ;
    assertEquals ( r , q ) ;
  }
  @Test public void comparablePairEqualToFalse ( ) {
    final ComparablePair<Integer, String>q = new ComparablePair<Integer, String> ( 20 , "blah" ) ;
    assertFalse ( p.equals ( q ) ) ;
    assertFalse ( q.equals ( p ) ) ;
  }
  @Test public void comparablePairEqualToFalseNulls ( ) {
    final ComparablePair<Integer, String>q = new ComparablePair<Integer, String> ( null , null ) ;
    assertFalse ( p.equals ( q ) ) ;
    assertFalse ( q.equals ( p ) ) ;
  }
  @Test public void comparablePairToString ( ) {
    assertEquals ( p.toString ( ) ,  "(" + i + ", " + s + ")" ) ;
  }
  @Test public void comparisonLessThan ( ) {
    final ComparablePair<Integer, String>q = new ComparablePair<Integer, String> ( 20 , "blah" ) ;
    assertEquals ( p.compareTo ( q ) , -1 ) ;
    assertEquals ( q.compareTo ( p ) , 1 ) ;
  }
  @Test public void comparisonEqual ( ) {
    assertEquals ( p.compareTo ( p ) , 0 ) ;
  }
  @Test public void comparisonGreaterThan ( ) {
    final ComparablePair<Integer, String>q = new ComparablePair<Integer, String> ( 0 , "blah" ) ;
    assertEquals ( p.compareTo ( q ) , 1 ) ;
    assertEquals ( q.compareTo ( p ) , -1 ) ;
  }

}
