/*
 *  ADS – A Java package of algorithms and data structures for educational purposes.  This software was
 *  originally created for the book Winder & Roberts, "Developing Java Software", Wiley.
 *
 *  Copyright © 1997–2008 Russel Winder
 *
 *  This file is part of ADS.
 *
 *  ADS is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General
 *  Public License as published by the Free Software Foundation, either version 3 of the License, or (at
 *  your option) any later version.
 *
 *  ADS is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
 *  implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General
 *  Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License along with ADS.  If not, see
 *  <http://www.gnu.org/licenses/>.
 */
package org.devjavasoft.ads.tests ;

import org.devjavasoft.ads.LessThanComparator ;
import org.devjavasoft.ads.LessThanOrEqualToComparator ;
import org.devjavasoft.ads.EqualToComparator ;
import org.devjavasoft.ads.NotEqualToComparator ;
import org.devjavasoft.ads.GreaterThanComparator ;
import org.devjavasoft.ads.GreaterThanOrEqualToComparator ;

import org.testng.annotations.Test ;
import static org.testng.Assert.assertTrue ;
import static org.testng.Assert.assertFalse ;

/**
 *  A class to test the generic order relation classes.
 *
 *  @author Russel Winder
 */
public class Comparator_Test {
  /**
   *  <code>Integer</code> variable A.  This variable must conform to java.lang.Comparable.
   */
  private final Integer aI = 1 ;
  /**
   *  <code>Integer</code> variable B.  This variable must conform to java.lang.Comparable.
   */
  private final Integer bI = 10 ;
  /**
   *  <code>String</code> variable A.  This variable must conform to java.lang.Comparable.
   */
  private final String aS = "a" ;
  /**
   *  <code>String</code> variable B.  This variable must conform to java.lang.Comparable.
   */
  private final String bS = "b" ;

  @Test public void lessThanInteger ( ) {
    assertTrue ( ( new LessThanComparator<Integer>  ( ) ).relation ( aI, bI ) ) ;
    assertTrue ( ( new LessThanOrEqualToComparator<Integer>  ( ) ).relation ( aI, bI ) ) ;
    assertFalse ( ( new EqualToComparator<Integer>  ( ) ).relation ( aI, bI ) ) ;
    assertTrue ( ( new NotEqualToComparator<Integer>  ( ) ).relation ( aI, bI ) ) ;
    assertFalse ( ( new GreaterThanComparator<Integer>  ( ) ).relation ( aI, bI ) ) ;
    assertFalse ( ( new GreaterThanOrEqualToComparator<Integer>  ( ) ).relation ( aI, bI ) ) ;
  }
  @Test public void greaterThanInteger ( ) {
    assertFalse ( ( new LessThanComparator<Integer>  ( ) ).relation ( bI, aI ) ) ;
    assertFalse ( ( new LessThanOrEqualToComparator<Integer>  ( ) ).relation ( bI, aI ) ) ;
    assertFalse ( ( new EqualToComparator<Integer>  ( ) ).relation ( bI, aI ) ) ;
    assertTrue ( ( new NotEqualToComparator<Integer>  ( ) ).relation ( bI, aI ) ) ;
    assertTrue ( ( new GreaterThanComparator<Integer>  ( ) ).relation ( bI, aI ) ) ;
    assertTrue ( ( new GreaterThanOrEqualToComparator<Integer>  ( ) ).relation ( bI, aI ) ) ;
  }
  @Test public void equalToInteger ( ) {
    assertFalse ( ( new LessThanComparator<Integer>  ( ) ).relation ( aI, aI ) ) ;
    assertTrue ( ( new LessThanOrEqualToComparator<Integer>  ( ) ).relation ( aI, aI ) ) ;
    assertTrue ( ( new EqualToComparator<Integer>  ( ) ).relation ( aI, aI ) ) ;
    assertFalse ( ( new NotEqualToComparator<Integer>  ( ) ).relation ( aI, aI ) ) ;
    assertFalse ( ( new GreaterThanComparator<Integer>  ( ) ).relation ( aI, aI ) ) ;
    assertTrue ( ( new GreaterThanOrEqualToComparator<Integer>  ( ) ).relation ( aI, aI ) ) ;
  }
  @Test public void lessThanString ( ) {
    assertTrue ( ( new LessThanComparator<String>  ( ) ).relation ( aS, bS ) ) ;
    assertTrue ( ( new LessThanOrEqualToComparator<String>  ( ) ).relation ( aS, bS ) ) ;
    assertFalse ( ( new EqualToComparator<String>  ( ) ).relation ( aS, bS ) ) ;
    assertTrue ( ( new NotEqualToComparator<String>  ( ) ).relation ( aS, bS ) ) ;
    assertFalse ( ( new GreaterThanComparator<String>  ( ) ).relation ( aS, bS ) ) ;
    assertFalse ( ( new GreaterThanOrEqualToComparator<String>  ( ) ).relation ( aS, bS ) ) ;
  }
  @Test public void greaterThanString ( ) {
    assertFalse ( ( new LessThanComparator<String>  ( ) ).relation ( bS, aS ) ) ;
    assertFalse ( ( new LessThanOrEqualToComparator<String>  ( ) ).relation ( bS, aS ) ) ;
    assertFalse ( ( new EqualToComparator<String>  ( ) ).relation ( bS, aS ) ) ;
    assertTrue ( ( new NotEqualToComparator<String>  ( ) ).relation ( bS, aS ) ) ;
    assertTrue ( ( new GreaterThanComparator<String>  ( ) ).relation ( bS, aS ) ) ;
    assertTrue ( ( new GreaterThanOrEqualToComparator<String>  ( ) ).relation ( bS, aS ) ) ;
  }
  @Test public void equalToString ( ) {
    assertFalse ( ( new LessThanComparator<String>  ( ) ).relation ( aS, aS ) ) ;
    assertTrue ( ( new LessThanOrEqualToComparator<String>  ( ) ).relation ( aS, aS ) ) ;
    assertTrue ( ( new EqualToComparator<String>  ( ) ).relation ( aS, aS ) ) ;
    assertFalse ( ( new NotEqualToComparator<String>  ( ) ).relation ( aS, aS ) ) ;
    assertFalse ( ( new GreaterThanComparator<String>  ( ) ).relation ( aS, aS ) ) ;
    assertTrue ( ( new GreaterThanOrEqualToComparator<String>  ( ) ).relation ( aS, aS ) ) ;
  }
  @Test public void comparatorEquals ( ) {
    assertFalse ( ( new LessThanComparator<Integer> ( ) ).equals ( new GreaterThanComparator<Integer> ( ) ) ) ;
    assertFalse ( ( new LessThanComparator<Integer> ( ) ).equals ( new GreaterThanOrEqualToComparator<Integer> ( ) ) ) ;
    assertFalse ( ( new LessThanComparator<Integer> ( ) ).equals ( new LessThanOrEqualToComparator<Integer> ( ) ) ) ;
    assertTrue ( ( new LessThanComparator<Integer> ( ) ).equals ( new LessThanComparator<Integer> ( ) ) ) ;
    assertFalse ( ( new LessThanComparator<Integer> ( ) ).equals ( new EqualToComparator<Integer> ( ) ) ) ;
    assertFalse ( ( new LessThanComparator<Integer> ( ) ).equals ( new NotEqualToComparator<Integer> ( ) ) ) ;
    assertFalse ( ( new LessThanOrEqualToComparator<Integer> ( ) ).equals ( new GreaterThanComparator<Integer> ( ) ) ) ;
    assertFalse ( ( new LessThanOrEqualToComparator<Integer> ( ) ).equals ( new GreaterThanOrEqualToComparator<Integer> ( ) ) ) ;
    assertTrue ( ( new LessThanOrEqualToComparator<Integer> ( ) ).equals ( new LessThanOrEqualToComparator<Integer> ( ) ) ) ;
    assertFalse ( ( new LessThanOrEqualToComparator<Integer> ( ) ).equals ( new LessThanComparator<Integer> ( ) ) ) ;
    assertFalse ( ( new LessThanOrEqualToComparator<Integer> ( ) ).equals ( new EqualToComparator<Integer> ( ) ) ) ;
    assertFalse ( ( new LessThanOrEqualToComparator<Integer> ( ) ).equals ( new NotEqualToComparator<Integer> ( ) ) ) ;
    assertTrue ( ( new GreaterThanComparator<Integer> ( ) ).equals ( new GreaterThanComparator<Integer> ( ) ) ) ;
    assertFalse ( ( new GreaterThanComparator<Integer> ( ) ).equals ( new GreaterThanOrEqualToComparator<Integer> ( ) ) ) ;
    assertFalse ( ( new GreaterThanComparator<Integer> ( ) ).equals ( new LessThanOrEqualToComparator<Integer> ( ) ) ) ;
    assertFalse ( ( new GreaterThanComparator<Integer> ( ) ).equals ( new LessThanComparator<Integer> ( ) ) ) ;
    assertFalse ( ( new GreaterThanComparator<Integer> ( ) ).equals ( new EqualToComparator<Integer> ( ) ) ) ;
    assertFalse ( ( new GreaterThanComparator<Integer> ( ) ).equals ( new NotEqualToComparator<Integer> ( ) ) ) ;
    assertFalse ( ( new GreaterThanOrEqualToComparator<Integer> ( ) ).equals ( new GreaterThanComparator<Integer> ( ) ) ) ;
    assertTrue ( ( new GreaterThanOrEqualToComparator<Integer> ( ) ).equals ( new GreaterThanOrEqualToComparator<Integer> ( ) ) ) ;
    assertFalse ( ( new GreaterThanOrEqualToComparator<Integer> ( ) ).equals ( new LessThanOrEqualToComparator<Integer> ( ) ) ) ;
    assertFalse ( ( new GreaterThanOrEqualToComparator<Integer> ( ) ).equals ( new LessThanComparator<Integer> ( ) ) ) ;
    assertFalse ( ( new GreaterThanOrEqualToComparator<Integer> ( ) ).equals ( new EqualToComparator<Integer> ( ) ) ) ;
    assertFalse ( ( new GreaterThanOrEqualToComparator<Integer> ( ) ).equals ( new NotEqualToComparator<Integer> ( ) ) ) ;
    assertFalse ( ( new EqualToComparator<Integer> ( ) ).equals ( new GreaterThanComparator<Integer> ( ) ) ) ;
    assertFalse ( ( new EqualToComparator<Integer> ( ) ).equals ( new GreaterThanOrEqualToComparator<Integer> ( ) ) ) ;
    assertFalse ( ( new EqualToComparator<Integer> ( ) ).equals ( new LessThanOrEqualToComparator<Integer> ( ) ) ) ;
    assertFalse ( ( new EqualToComparator<Integer> ( ) ).equals ( new LessThanComparator<Integer> ( ) ) ) ;
    assertTrue ( ( new EqualToComparator<Integer> ( ) ).equals ( new EqualToComparator<Integer> ( ) ) ) ;
    assertFalse ( ( new EqualToComparator<Integer> ( ) ).equals ( new NotEqualToComparator<Integer> ( ) ) ) ;
    assertFalse ( ( new NotEqualToComparator<Integer> ( ) ).equals ( new GreaterThanComparator<Integer> ( ) ) ) ;
    assertFalse ( ( new NotEqualToComparator<Integer> ( ) ).equals ( new GreaterThanOrEqualToComparator<Integer> ( ) ) ) ;
    assertFalse ( ( new NotEqualToComparator<Integer> ( ) ).equals ( new LessThanOrEqualToComparator<Integer> ( ) ) ) ;
    assertFalse ( ( new NotEqualToComparator<Integer> ( ) ).equals ( new LessThanComparator<Integer> ( ) ) ) ;
    assertFalse ( ( new NotEqualToComparator<Integer> ( ) ).equals ( new EqualToComparator<Integer> ( ) ) ) ;
    assertTrue ( ( new NotEqualToComparator<Integer> ( ) ).equals ( new NotEqualToComparator<Integer> ( ) ) ) ;
  }
  /*
   *  Due to erasure X<A> and X<B> are the same type at run time in Java (unlike any other programming
   *  language that has parameterized types).  Given that they are different types at compile time this
   *  leads to some seriously confused programmers.  The upshot is that the assertions below seem to be the
   *  wrong way round from a type theory perspective but they are the right way round from a Java
   *  perspective.
   */
  @Test public void comparatorRuntimeType ( ) {
    assertTrue ( ( new LessThanComparator<Integer> ( ) ).equals ( new LessThanComparator<String> ( ) ) ) ;
    assertTrue ( ( new LessThanOrEqualToComparator<Integer> ( ) ).equals ( new LessThanOrEqualToComparator<String> ( ) ) ) ;
    assertTrue ( ( new GreaterThanComparator<Integer> ( ) ).equals ( new GreaterThanComparator<String> ( ) ) ) ;
    assertTrue ( ( new EqualToComparator<Integer> ( ) ).equals ( new EqualToComparator<String> ( ) ) ) ;
    assertTrue ( ( new GreaterThanOrEqualToComparator<Integer> ( ) ).equals ( new GreaterThanOrEqualToComparator<String> ( ) ) ) ;
    assertTrue ( ( new NotEqualToComparator<Integer> ( ) ).equals ( new NotEqualToComparator<String> ( ) ) ) ;
  }
}
