/*
 *  ADS – A Java package of algorithms and data structures for educational purposes.  This software was
 *  originally created for the book Winder & Roberts, "Developing Java Software", Wiley.
 *
 *  Copyright © 1997–2008,2011 Russel Winder
 *
 *  This file is part of ADS.
 *
 *  ADS is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General
 *  Public License as published by the Free Software Foundation, either version 3 of the License, or (at
 *  your option) any later version.
 *
 *  ADS is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
 *  implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General
 *  Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License along with ADS.  If not, see
 *  <http://www.gnu.org/licenses/>.
 */
package org.devjavasoft.ads.tests ;

import org.devjavasoft.ads.Array ;
import org.devjavasoft.ads.DLList ;
import org.devjavasoft.ads.Comparator ;
import org.devjavasoft.ads.LessThanComparator ;
import org.devjavasoft.ads.Heap ;

import org.testng.annotations.Test ;
import static org.testng.Assert.assertTrue ;

/**
 *  A class to test out the <code>Heap&lt;T></code> class.
 *
 *  @author Russel Winder
 */
public class Heap_Test {
  private final Integer[] unsortedPrimitiveArray = { 3 , 56 , 2 , 33 , 38 , 23 } ;
  private final Integer[] sortedPrimitiveArray = { 2 , 3 , 23 , 33 , 38 , 56 } ;
  private final Comparator<Integer> lessThan = new LessThanComparator<Integer> ( ) ;
  @Test public void initializationSorted ( ) {
    final Array<Integer> array = new Array<Integer> ( ) ; for ( final Integer i : sortedPrimitiveArray ) { array.add ( i ) ; }
    final DLList<Integer> dllist = new DLList<Integer> ( array ) ;
    final Heap<Integer> a = new Heap<Integer> ( sortedPrimitiveArray , lessThan ) ;
    final Heap<Integer> b = new Heap<Integer> ( array , lessThan ) ;
    final Heap<Integer> c = new Heap<Integer> ( dllist , lessThan ) ;
    final Heap<Integer> d = new Heap.Synchronized<Integer> ( new Heap<Integer> ( sortedPrimitiveArray , lessThan ) ) ;
    final Heap<Integer> e = new Heap.Synchronized<Integer> ( new Heap<Integer> ( array , lessThan ) ) ;
    final Heap<Integer> f = new Heap.Synchronized<Integer> ( new Heap<Integer> ( dllist , lessThan ) ) ;
    runNonDestructiveComparisons ( a , b , c , d , e , f ) ;
    runDestructiveTest ( a ) ;
    runDestructiveTest ( b ) ;
    runDestructiveTest ( c ) ;
    runDestructiveTest ( d ) ;
    runDestructiveTest ( e ) ;
    runDestructiveTest ( f ) ;
  }
  @Test public void initializationUnsorted ( ) {
    final Array<Integer> array = new Array<Integer> ( ) ; for ( Integer i : unsortedPrimitiveArray ) { array.add ( i ) ; }
    final DLList<Integer> dllist = new DLList<Integer> ( array ) ;
    final Heap<Integer> a = new Heap<Integer> ( unsortedPrimitiveArray , lessThan ) ;
    final Heap<Integer> b = new Heap<Integer> ( array , lessThan ) ;
    final Heap<Integer> c = new Heap<Integer> ( dllist , lessThan ) ;
    final Heap<Integer> d = new Heap.Synchronized<Integer> ( new Heap<Integer> ( unsortedPrimitiveArray , lessThan ) ) ;
    final Heap<Integer> e = new Heap.Synchronized<Integer> ( new Heap<Integer> ( array , lessThan ) ) ;
    final Heap<Integer> f = new Heap.Synchronized<Integer> ( new Heap<Integer> ( dllist , lessThan ) ) ;
    runNonDestructiveComparisons ( a , b , c , d , e , f ) ;
    runDestructiveTest ( a ) ;
    runDestructiveTest ( b ) ;
    runDestructiveTest ( c ) ;
    runDestructiveTest ( d ) ;
    runDestructiveTest ( e ) ;
    runDestructiveTest ( f ) ;
  }
  @Test public void mixedActivity ( ) {
    final Heap<Integer> x = new Heap<Integer> ( sortedPrimitiveArray , lessThan ) ;
    assertTrue ( x.remove ( ).equals ( 2 ) ) ;
    x.add ( 3 ) ;
    x.add ( 40 ) ;
    assertTrue ( x.remove ( ).equals ( 3 ) ) ;
    assertTrue ( x.remove ( ).equals ( 3 ) ) ;
    assertTrue ( x.remove ( ).equals ( 23 ) ) ;
    assertTrue ( x.remove ( ).equals ( 33 ) ) ;
    assertTrue ( x.remove ( ).equals ( 38 ) ) ;
    assertTrue ( x.remove ( ).equals ( 40 ) ) ;
    assertTrue ( x.remove ( ).equals ( 56 ) ) ;
  }
  @Test public void priorityQueueProblem ( ) {
    final Heap<Integer> x = new Heap<Integer> ( lessThan ) ;
    x.add ( 1 ) ;
    x.add ( 2 ) ;
    x.add ( 3 ) ;
    x.add ( 4 ) ;
    x.add ( 5 ) ;
    assertTrue ( x.remove ( ).equals ( 1 ) ) ;
    assertTrue ( x.remove ( ).equals ( 2 ) ) ;
    assertTrue ( x.remove ( ).equals ( 3 ) ) ;
    assertTrue ( x.remove ( ).equals ( 4 ) ) ;
    assertTrue ( x.remove ( ).equals ( 5 ) ) ;
  }
  private void runNonDestructiveComparisons ( final Heap<Integer> a , final Heap<Integer> b , final Heap<Integer> c , final Heap<Integer> d , final Heap<Integer> e , final Heap<Integer> f ) {
    assertTrue ( a.size ( ) == sortedPrimitiveArray.length ) ;
    assertTrue ( b.size ( ) == sortedPrimitiveArray.length ) ;
    assertTrue ( c.size ( ) == sortedPrimitiveArray.length ) ;
    assertTrue ( d.size ( ) == sortedPrimitiveArray.length ) ;
    assertTrue ( e.size ( ) == sortedPrimitiveArray.length ) ;
    assertTrue ( f.size ( ) == sortedPrimitiveArray.length ) ;
    assertTrue ( a.equals ( a ) ) ;
    assertTrue ( a.equals ( b ) ) ;
    assertTrue ( a.equals ( c ) ) ;
    assertTrue ( a.equals ( d ) ) ;
    assertTrue ( a.equals ( e ) ) ;
    assertTrue ( a.equals ( f ) ) ;
    assertTrue ( b.equals ( a ) ) ;
    assertTrue ( b.equals ( b ) ) ;
    assertTrue ( b.equals ( c ) ) ;
    assertTrue ( b.equals ( d ) ) ;
    assertTrue ( b.equals ( e ) ) ;
    assertTrue ( b.equals ( f ) ) ;
    assertTrue ( c.equals ( a ) ) ;
    assertTrue ( c.equals ( b ) ) ;
    assertTrue ( c.equals ( c ) ) ;
    assertTrue ( c.equals ( d ) ) ;
    assertTrue ( c.equals ( e ) ) ;
    assertTrue ( c.equals ( f ) ) ;
    assertTrue ( d.equals ( a ) ) ;
    assertTrue ( d.equals ( b ) ) ;
    assertTrue ( d.equals ( c ) ) ;
    assertTrue ( d.equals ( d ) ) ;
    assertTrue ( d.equals ( e ) ) ;
    assertTrue ( d.equals ( f ) ) ;
    assertTrue ( e.equals ( a ) ) ;
    assertTrue ( e.equals ( b ) ) ;
    assertTrue ( e.equals ( c ) ) ;
    assertTrue ( e.equals ( d ) ) ;
    assertTrue ( e.equals ( e ) ) ;
    assertTrue ( e.equals ( f ) ) ;
    assertTrue ( f.equals ( a ) ) ;
    assertTrue ( f.equals ( b ) ) ;
    assertTrue ( f.equals ( c ) ) ;
    assertTrue ( f.equals ( d ) ) ;
    assertTrue ( f.equals ( e ) ) ;
    assertTrue ( f.equals ( f ) ) ;
  }
  private void runDestructiveTest ( final Heap<Integer> h ) {
    for ( final Integer item : sortedPrimitiveArray ) {
      assertTrue ( h.remove ( ).equals ( item ) ) ;
    }
    assertTrue ( h.isEmpty ( ) ) ;
  }
}
