/*
 *  ADS – A Java package of algorithms and data structures for educational purposes.  This software was
 *  originally created for the book Winder & Roberts, "Developing Java Software", Wiley.
 *
 *  Copyright © 1997–2009 Russel Winder
 *
 *  This file is part of ADS.
 *
 *  ADS is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General
 *  Public License as published by the Free Software Foundation, either version 3 of the License, or (at
 *  your option) any later version.
 *
 *  ADS is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
 *  implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General
 *  Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License along with ADS.  If not, see
 *  <http://www.gnu.org/licenses/>.
 */
package org.devjavasoft.ads.tests ;

import org.devjavasoft.ads.Array ;
import org.devjavasoft.ads.Comparator ;
import org.devjavasoft.ads.DLList ;
import org.devjavasoft.ads.EqualToComparator ;
import org.devjavasoft.ads.InputIterator ;
import org.devjavasoft.ads.LessThanComparator ;
import org.devjavasoft.ads.RandomAccessIterator ;
import org.devjavasoft.ads.searching.LinearSearch ;
import org.devjavasoft.ads.searching.BinaryChopSearch ;

import org.testng.annotations.Test ;
import static org.testng.Assert.assertTrue ;

/**
 *  A class to test the various searching methods using JUnit.
 *
 *  @author Russel Winder
 */
public class Searching_Test  {
  //  Data must be sorted so as to support binary chop searching.
  private final Integer[] data = { 1 , 2 , 3 , 4 , 5 , 6 , 7 , 8 , 9 , 10 , 11 } ;
  private final Array<Integer> array = new Array<Integer> ( ) ; {
    for ( final Integer i : data ) { array.add ( i ) ; }
  }
  private final DLList<Integer> list = new DLList<Integer> ( ) ; {
    for ( final Integer i : data ) { list.add ( i ) ; }
  }
  private final Comparator<Integer> ce = new EqualToComparator<Integer>  ( ) ;
  private final Comparator<Integer> cr = new LessThanComparator<Integer>  ( ) ;

  //  NB  Note the way that the iterator related assertions are tested -- the search method can return null.

  @Test public void linearSearchPrimitiveArray ( ) {
    assertTrue ( LinearSearch.search ( data , 3  , ce ) == 2 ) ;
    assertTrue ( LinearSearch.search ( data , 9  , ce ) == 8 ) ;
  }
  @Test public void linearSearchArray ( ) {
    final InputIterator<Integer> positionOfThree = array.begin ( ) ;
    positionOfThree.advance ( 2 ) ;
    final InputIterator<Integer> positionOfNine = array.begin ( ) ;
    positionOfNine.advance ( 8 ) ;
    assertTrue ( positionOfThree.equals ( LinearSearch.search ( array , 3 , ce ) ) ) ;
    assertTrue ( positionOfNine.equals ( LinearSearch.search ( array , 9 , ce ) ) ) ;
  }
  @Test public void linearSearchArrayIterators ( ) {
    final InputIterator<Integer> positionOfThree = array.begin ( ) ;
    positionOfThree.advance ( 2 ) ;
    final InputIterator<Integer> positionOfNine = array.begin ( ) ;
    positionOfNine.advance ( 8 ) ;
    assertTrue ( positionOfThree.equals ( LinearSearch.search ( array.begin ( ) , array.end ( ) , 3 , ce ) ) ) ;
    assertTrue ( positionOfNine.equals ( LinearSearch.search ( array.begin ( ) , array.end ( ) , 9 , ce ) ) ) ;
  }
  @Test public void linearSearchDLList ( ) {
    final InputIterator<Integer> positionOfThree = list.begin ( ) ;
    positionOfThree.advance ( 2 ) ;
    final InputIterator<Integer> positionOfNine = list.begin ( ) ;
    positionOfNine.advance ( 8 ) ;
    assertTrue ( positionOfThree.equals ( LinearSearch.search ( list , 3 , ce ) ) ) ;
    assertTrue ( positionOfNine.equals ( LinearSearch.search ( list , 9 , ce ) ) ) ;
  }
  @Test public void linearSearchDLListIterators ( ) {
    final InputIterator<Integer> positionOfThree = list.begin ( ) ;
    positionOfThree.advance ( 2 ) ;
    final InputIterator<Integer> positionOfNine = list.begin ( ) ;
    positionOfNine.advance ( 8 ) ;
    assertTrue ( positionOfThree.equals ( LinearSearch.search ( list.begin ( ) , list.end ( ) , 3 , ce ) ) ) ;
    assertTrue ( positionOfNine.equals ( LinearSearch.search ( list.begin ( ) , list.end ( ) , 9 , ce ) ) ) ;
  }
  @Test public void binaryChopSearchPrimitiveArray ( ) {
    assertTrue ( BinaryChopSearch.search ( data , 3  , ce , cr ) == 2 ) ;
    assertTrue ( BinaryChopSearch.search ( data , 9  , ce , cr ) == 8 ) ;
  }
  @Test public void binaryChopSearchArray ( ) {
    final RandomAccessIterator<Integer> positionOfThree = array.begin ( ) ;
    positionOfThree.advance ( 2 ) ;
    final RandomAccessIterator<Integer> positionOfNine = array.begin ( ) ;
    positionOfNine.advance ( 8 ) ;
    assertTrue ( positionOfThree.equals ( BinaryChopSearch.search ( array , 3 , ce , cr ) ) ) ;
    assertTrue ( positionOfNine.equals ( BinaryChopSearch.search ( array , 9 , ce , cr ) ) ) ;
  }
  @Test public void binaryChopSearchArrayIterators ( ) {
    final RandomAccessIterator<Integer> positionOfThree = array.begin ( ) ;
    positionOfThree.advance ( 2 ) ;
    final RandomAccessIterator<Integer> positionOfNine = array.begin ( ) ;
    positionOfNine.advance ( 8 ) ;
    assertTrue ( positionOfThree.equals ( BinaryChopSearch.search ( array.begin ( ) , array.end ( ) , 3 , ce , cr ) ) ) ;
    assertTrue ( positionOfNine.equals ( BinaryChopSearch.search ( array.begin ( ) , array.end ( ) , 9 , ce , cr ) ) ) ;
  }
}
