/*
 *  ADS – A Java package of algorithms and data structures for educational purposes.  This software was
 *  originally created for the book Winder & Roberts, "Developing Java Software", Wiley.
 *
 *  Copyright © 1997–2008,2010 Russel Winder
 *
 *  This file is part of ADS.
 *
 *  ADS is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General
 *  Public License as published by the Free Software Foundation, either version 3 of the License, or (at
 *  your option) any later version.
 *
 *  ADS is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
 *  implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General
 *  Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License along with ADS.  If not, see
 *  <http://www.gnu.org/licenses/>.
 */
package org.devjavasoft.ads.tests ;

import org.devjavasoft.ads.OrderedBinaryTreeSet ;
//import org.devjavasoft.ads.OpenHashTableSet ;
//import org.devjavasoft.ads.ChainedHashTableSet ;
import org.devjavasoft.ads.LessThanComparator ;
import org.devjavasoft.ads.Set ;

import org.testng.annotations.BeforeMethod ;
import org.testng.annotations.Test ;
import static org.testng.Assert.assertTrue ;
import static org.testng.Assert.assertFalse ;

/**
 *  @author Russel Winder
 */
public class Set_Test {
  /**
   *  A set which will use an <code>OrderedBinaryTreeSet&lt;String></code>.
   */
  Set<String> a ;
  /**
   *  A set which will use an <code>OrderedBinaryTreeSet&lt;String></code>.
   */
  Set<String> aa ;
  /**
   *  A set which will use an <code>OrderedBinaryTreeSet&lt;String></code>.
   */
  Set<String> aaa ;
  /**
   *  A set which will use an <code>OpenHashSet&lt;String></code>.
   */
  Set<String> b ;
  /**
   *  A set which will use an <code>ChainedHashSet&lt;String></code>.
   */
  Set<String> c ;
  /**
   *  The set up method.
   */
  @BeforeMethod  public void setUp ( ) {
    a = new OrderedBinaryTreeSet<String> ( new LessThanComparator<String> ( ) ) ;
    //b = new OpenHashTableSet<String> ( ) ;
    //c = new Set.Synchronized<String> ( new ChainedHashTableSet<String> ( ) ) ;
    fillOut ( a ) ;
    //fillOut ( b ) ;
    //fillOut ( c ) ;
  }
  @Test public void testClone ( ) {
    aa = a.clone ( ) ;
    assertTrue ( aa.equals ( a ) ) ;
    assertTrue ( a.equals ( aa ) ) ;
  }
  @Test public void remove ( ) {
    a.remove ( "punk" ) ;
    assertFalse ( a.contains ( "punk" ) ) ;
  }
  @Test public void union ( ) {
    aa = a.clone ( )  ;
    aa.remove ( "Groovy" ) ;
    aa.remove ( "Gudren" ) ;
    aa.remove ( "Hi" ) ;
    aa.remove ( "Ho" ) ;
    aaa = a.clone ( )  ;
    aaa.remove ( "punk" ) ;
    aaa.remove ( "whack" ) ;
    aaa.remove ( "sick" ) ;
    assertFalse ( aa.equals ( aaa ) ) ;
    assertFalse ( aaa.equals ( aa ) ) ;
    assertTrue ( aa.union ( aaa ).equals ( a ) ) ;
  }
  @Test public void intersection ( ) {
    aa = a.clone ( )  ;
    aa.remove ( "Groovy" ) ;
    aa.remove ( "Gudren" ) ;
    aa.remove ( "Hi" ) ;
    aa.remove ( "Ho" ) ;
    aaa = a.clone ( )  ;
    aaa.remove ( "punk" ) ;
    aaa.remove ( "whack" ) ;
    aaa.remove ( "sick" ) ;
    assertFalse ( aa.equals ( aaa ) ) ;
    assertFalse ( aaa.equals ( aa ) ) ;
    Set<String> z = new OrderedBinaryTreeSet<String> ( new LessThanComparator<String>  ( ) ) ;
    z.add ( "Hello" ) ;
    assertFalse ( aa.intersection ( aaa ).equals ( a ) ) ;
    assertTrue ( aa.intersection ( aaa ).equals ( z ) ) ;
  }
  @Test public void difference ( ) {
    aa = a.clone ( )  ;
    aa.remove ( "Groovy" ) ;
    aa.remove ( "Gudren" ) ;
    aa.remove ( "Hi" ) ;
    aa.remove ( "Ho" ) ;
    aaa = a.clone ( )  ;
    aaa.remove ( "punk" ) ;
    aaa.remove ( "whack" ) ;
    aaa.remove ( "sick" ) ;
    assertFalse ( aa.equals ( aaa ) ) ;
    assertFalse ( aaa.equals ( aa ) ) ;
    Set<String> z = new OrderedBinaryTreeSet<String> ( new LessThanComparator<String>  ( ) ) ;
    z.add ( "punk" ) ;
    z.add ( "sick" ) ;
    z.add ( "whack" ) ;
    assertFalse ( aa.difference ( aaa ).equals ( a ) ) ;
    assertTrue ( aa.difference ( aaa ).equals ( z ) ) ;
  }
  private void fillOut ( final Set<String> s ) {
    s.add ( "Groovy" ) ;
    s.add ( "Gudren" ) ;
    s.add ( "Hello" ) ;
    s.add ( "Hi" ) ;
    s.add ( "Ho" ) ;
    s.add ( "punk" ) ;
    s.add ( "sick" ) ;
    s.add ( "whack" ) ;
    s.add ( "Hello" ) ;
    s.add ( "Groovy" ) ;
    s.add ( "Ho" ) ;
    s.add ( "sick" ) ;
    s.add ( "Gudren" ) ;
    s.add ( "Hi" ) ;
    s.add ( "punk" ) ;
    s.add ( "whack" ) ;
  }
}
