/*
 *  ADS – A Java package of algorithms and data structures for educational purposes.  This software was
 *  originally created for the book Winder & Roberts, "Developing Java Software", Wiley.
 *
 *  Copyright © 1997–2008 Russel Winder
 *
 *  This file is part of ADS.
 *
 *  ADS is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General
 *  Public License as published by the Free Software Foundation, either version 3 of the License, or (at
 *  your option) any later version.
 *
 *  ADS is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
 *  implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General
 *  Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License along with ADS.  If not, see
 *  <http://www.gnu.org/licenses/>.
 */
package org.devjavasoft.ads.tests ;

import org.devjavasoft.ads.Array ;
import org.devjavasoft.ads.Comparator ;
import org.devjavasoft.ads.DLList ;
import org.devjavasoft.ads.EqualToComparator ;
import org.devjavasoft.ads.LessThanComparator ;
import org.devjavasoft.ads.sorting.SelectionSort ;
import org.devjavasoft.ads.sorting.BubbleSort ;
import org.devjavasoft.ads.sorting.InsertionSortBasic ;
import org.devjavasoft.ads.sorting.InsertionSort ;
import org.devjavasoft.ads.sorting.HeapSort ;
import org.devjavasoft.ads.sorting.HeapSortArray ;
import org.devjavasoft.ads.sorting.Quicksort ;
import org.devjavasoft.ads.sorting.Quicksort2 ;
import org.devjavasoft.ads.sorting.Quicksort3 ;
import org.devjavasoft.ads.sorting.Quicksort4 ;
import org.devjavasoft.ads.sorting.QuicksortSequence ;
import org.devjavasoft.ads.sorting.MergeSort ;
import org.devjavasoft.ads.sorting.CountSort ;
import org.devjavasoft.ads.sorting.RadixSort ;
import org.devjavasoft.ads.sorting.BinaryTreeSort ;

import java.util.Arrays ;

import org.testng.annotations.BeforeMethod ;
import org.testng.annotations.Test ;
import static org.testng.Assert.assertTrue ;

/**
 *  A class to test the various sorting methods.
 *
 *  @author Russel Winder
 */
public class Sorting_Test {
  private final Integer[] unsortedData = new Integer[] {
     new Integer ( 11 ) ,
     new Integer ( 34 ) ,
     new Integer ( 67 ) ,
     new Integer ( 34 ) ,
     new Integer ( 56 ) ,
     new Integer ( 24 ) ,
     new Integer ( 51 ) ,
     new Integer ( 101 ) ,
     new Integer ( 1 ) ,
     new Integer ( 11 ) ,
     new Integer ( 34 ) ,
     new Integer ( 67 ) ,
     new Integer ( 34 ) ,
     new Integer ( 56 ) ,
     new Integer ( 24 ) ,
     new Integer ( 51 ) ,
     new Integer ( 101 ) ,
     new Integer ( 1 ) ,
     new Integer ( 11 ) ,
     new Integer ( 34 ) ,
     new Integer ( 67 ) ,
     new Integer ( 34 ) ,
     new Integer ( 56 ) ,
     new Integer ( 24 ) ,
     new Integer ( 51 ) ,
     new Integer ( 101 ) ,
     new Integer ( 1 )
  } ;
  private final Array<Integer> unsortedDataArray = new Array<Integer> ( ) ; {
    for ( Integer i : unsortedData ) { unsortedDataArray.add ( i ) ; }
  }
  private final DLList<Integer> unsortedDataDLList = new DLList<Integer> ( unsortedDataArray ) ;
  private Integer[] sortedData = new Integer [ unsortedData.length ] ; {
    //  Create an array to store the sorted data.  Copy the references from the unsorted data array
    //  and then sort using the sort method that comes standard with the Java Platform.  This assumes
    //  that the standard method actually works!
    System.arraycopy (  unsortedData , 0 , sortedData , 0 , unsortedData.length ) ;
    Arrays.sort ( sortedData ) ;
  } ;
  private final Array<Integer> sortedDataArray = new Array<Integer> ( ) ; {
    for ( Integer i : sortedData ) { sortedDataArray.add ( i ) ; }
  }
  private final DLList<Integer> sortedDataDLList = new DLList<Integer> ( sortedDataArray ) ;

  final Comparator<Integer> equalTo = new EqualToComparator<Integer> ( ) ;
  final Comparator<Integer> lessThan = new LessThanComparator<Integer> ( ) ;

  Integer[] data ;
  Array<Integer> dataArray ;
  DLList<Integer> dataDLList ;

  @BeforeMethod  public void setUp ( ) {
    data = unsortedData.clone ( ) ;
    dataArray = unsortedDataArray.clone ( ) ;
    dataDLList = unsortedDataDLList.clone ( ) ;
  }
  @Test public void selectionSort ( ) {
    SelectionSort.sort ( data , lessThan ) ;
    assertTrue ( Arrays.equals ( data, sortedData ) ) ;
  }
  @Test public void selectionSortArray ( ) {
    SelectionSort.sort ( dataArray , lessThan ) ;
    assertTrue ( dataArray.equals ( sortedDataArray ) ) ;
  }
  @Test public void selectionSortDLList ( ) {
    SelectionSort.sort ( dataDLList , lessThan ) ;
    assertTrue ( dataDLList.equals ( sortedDataDLList ) ) ;
  }
  @Test public void bubbleSort ( ) {
    BubbleSort.sort ( data , lessThan ) ;
    assertTrue ( Arrays.equals ( data, sortedData ) ) ;
  }
  @Test public void bubbleSortArray ( ) {
    BubbleSort.sort ( dataArray , lessThan ) ;
    assertTrue ( dataArray.equals ( sortedDataArray ) ) ;
  }
  @Test public void bubbleSortDLList ( ) {
    BubbleSort.sort ( dataDLList , lessThan ) ;
    assertTrue ( dataDLList.equals ( sortedDataDLList ) ) ;
  }
  @Test public void insertionSortBasic ( ) {
    InsertionSortBasic.sort ( data , lessThan ) ;
    assertTrue ( Arrays.equals ( data, sortedData ) ) ;
  }
  @Test public void insertionSort ( ) {
    InsertionSort.sort ( data , lessThan ) ;
    assertTrue ( Arrays.equals ( data, sortedData ) ) ;
  }
  @Test public void insertionSortArray ( ) {
    InsertionSort.sort ( dataArray , lessThan ) ;
    assertTrue ( dataArray.equals ( sortedDataArray ) ) ;
  }
  @Test public void insertionSortDLList ( ) {
    InsertionSort.sort ( dataDLList , lessThan ) ;
    assertTrue ( dataDLList.equals ( sortedDataDLList ) ) ;
  }
  @Test public void mergeSort ( ) {
    MergeSort.sort ( data , lessThan ) ;
    assertTrue ( Arrays.equals ( data, sortedData ) ) ;
  }
  @Test public void mergeSortArray ( ) {
    MergeSort.sort ( dataArray , lessThan ) ;
    assertTrue ( dataArray.equals ( sortedDataArray ) ) ;
  }
  @Test public void mergeSortDLList ( ) {
    MergeSort.sort ( dataDLList , lessThan ) ;
    assertTrue ( dataDLList.equals ( sortedDataDLList ) ) ;
  }
  @Test public void heapSort ( ) {
    HeapSort.sort ( data , lessThan ) ;
    assertTrue ( Arrays.equals ( data, sortedData ) ) ;
  }
  @Test public void heapSortArray ( ) {
    HeapSort.sort ( dataArray , lessThan ) ;
    assertTrue ( dataArray.equals ( sortedDataArray ) ) ;
  }
  @Test public void heapSortDLList ( ) {
    HeapSort.sort ( dataDLList , lessThan ) ;
    assertTrue ( dataDLList.equals ( sortedDataDLList ) ) ;
  }
  @Test public void heapSortArrayAlt ( ) {
    HeapSortArray.sort ( dataArray , lessThan ) ;
    assertTrue ( dataArray.equals ( sortedDataArray ) ) ;
  }
  @Test public void quicksort ( ) {
    Quicksort.sort ( data , lessThan ) ;
    assertTrue ( Arrays.equals ( data, sortedData ) ) ;
  }
  @Test public void quicksortArray ( ) {
    Quicksort.sort ( dataArray , lessThan , equalTo ) ;
    assertTrue ( dataArray.equals ( sortedDataArray ) ) ;
  }
  @Test public void quicksort2 ( ) {
    Quicksort2.sort ( data , lessThan , equalTo ) ;
    assertTrue ( Arrays.equals ( data, sortedData ) ) ;
  }
  @Test public void quicksort3 ( ) {
    Quicksort3.sort ( data , lessThan , equalTo ) ;
    assertTrue ( Arrays.equals ( data, sortedData ) ) ;
  }
  @Test public void quicksort4 ( ) {
    Quicksort4.sort ( data , lessThan , equalTo ) ;
    assertTrue ( Arrays.equals ( data, sortedData ) ) ;
  }
  @Test public void quicksortSequence ( ) {
    QuicksortSequence.sort ( dataArray , lessThan ) ;
    assertTrue ( dataArray.equals ( sortedDataArray ) ) ;
  }
  @Test public void quicksortDLList ( ) {
    Quicksort.sort ( dataDLList , lessThan , equalTo ) ;
    assertTrue ( dataDLList.equals ( sortedDataDLList ) ) ;
  }
  @Test public void countSort ( ) {
    CountSort.sort ( data ) ;
    assertTrue ( Arrays.equals ( data, sortedData ) ) ;
  }
  @Test public void countSortArray ( ) {
    CountSort.sort ( dataArray ) ;
    assertTrue ( dataArray.equals ( sortedDataArray ) ) ;
  }
  @Test public void countSortDLList ( ) {
    CountSort.sort ( dataDLList ) ;
    assertTrue ( dataDLList.equals ( sortedDataDLList ) ) ;
  }
  @Test public void radixSort ( ) {
    RadixSort.sort ( data , 3 , 10 ) ;
    assertTrue ( Arrays.equals ( data, sortedData ) ) ;
  }
  @Test public void radixSortArray ( ) {
    RadixSort.sort ( dataArray , 3 , 10 ) ;
    assertTrue ( dataArray.equals ( sortedDataArray ) ) ;
  }
  @Test public void radixSortDLList ( ) {
    RadixSort.sort ( dataDLList , 3 , 10 ) ;
    assertTrue ( dataDLList.equals ( sortedDataDLList ) ) ;
  }
  @Test public void binaryTreeSort ( ) {
    BinaryTreeSort.sort ( data , lessThan ) ;
    assertTrue ( Arrays.equals ( data, sortedData ) ) ;
  }
  @Test public void binaryTreeSortArray ( ) {
    BinaryTreeSort.sort ( dataArray , lessThan ) ;
    assertTrue ( dataArray.equals ( sortedDataArray ) ) ;
  }
  @Test public void binaryTreeSortDLList ( ) {
    BinaryTreeSort.sort ( dataDLList , lessThan ) ;
    assertTrue ( dataDLList.equals ( sortedDataDLList ) ) ;
  }
}
