/*
 *  ADS – A Java package of algorithms and data structures for educational purposes.  This software was
 *  originally created for the book Winder & Roberts, "Developing Java Software", Wiley.
 *
 *  Copyright © 1997–2008,2010 Russel Winder
 *
 *  This file is part of ADS.
 *
 *  ADS is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General
 *  Public License as published by the Free Software Foundation, either version 3 of the License, or (at
 *  your option) any later version.
 *
 *  ADS is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
 *  implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General
 *  Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License along with ADS.  If not, see
 *  <http://www.gnu.org/licenses/>.
 */
package org.devjavasoft.ads.tests ;

import org.devjavasoft.ads.InputIterator ;
import org.devjavasoft.ads.LessThanComparator ;
import org.devjavasoft.ads.BinaryTree ;
import org.devjavasoft.ads.OrderedBinaryTree ;
//import org.devjavasoft.ads.AVLTree ;

import org.testng.annotations.BeforeMethod ;
import org.testng.annotations.Test ;
import static org.testng.Assert.assertEquals ;

/**
 *  @author Russel Winder
 */
public class TreeIterator_Test {
  /**
   *  An <code>OrderedBinaryTree&lt;String></code> to iterate over.
   */
  OrderedBinaryTree<String> a ;
  /**
   *   The setup method which gets executed prior to any test method and must set up any variables needed by
   *   the tests.
   *
   *   <p>We are assuming that <code>Array<Integer>.add</code> completes execution even if it is not
   *   actually correct.</p>
   */
  @BeforeMethod  public void setUp ( ) {
    a = new OrderedBinaryTree<String> ( new LessThanComparator<String> ( ) ) ;
    fillOut(a) ;
  }
  /**
   */
  @Test public void preOrder ( ) {
    final String s = "Alpha Beta Delta Eta alpha beta delta eta " ;
    final StringBuilder sb = new StringBuilder () ;
    for ( InputIterator<String> i = a.preorderTraversal ( ) ; ! i.atEnd ( ) ; i.advance ( ) ) {
      sb.append ( i.get ( ) + " " ) ;
    }
    assertEquals ( sb.toString ( ) , s ) ;
  }
  /**
   */
  @Test public void postOrder ( ) {
    final String s = "eta delta beta alpha Eta Delta Beta Alpha " ;
    final StringBuilder sb = new StringBuilder ( ) ;
    for ( InputIterator<String> i = a.postorderTraversal ( ) ; ! i.atEnd ( ) ; i.advance ( ) ) {
      sb.append ( i.get ( ) + " " ) ;
    }
    assertEquals ( sb.toString ( ) , s ) ;
  }
  /**
   */
  @Test public void inOrder ( ) {
    final String s = "Alpha Beta Delta Eta alpha beta delta eta " ;
    final StringBuilder sb = new StringBuilder ( ) ;
    for ( InputIterator<String> i = a.inorderTraversal ( ) ; ! i.atEnd ( ) ; i.advance ( ) ) {
      sb.append ( i.get ( ) + " " ) ;
    }
    assertEquals ( sb.toString ( ) , s ) ;
  }
  /**
   */
  @Test public void levelOrder ( ) {
    final String s = "Alpha Beta Delta Eta alpha beta delta eta " ;
    final StringBuilder sb = new StringBuilder ( ) ;
    for ( InputIterator<String> i = a.levelorderTraversal ( ) ; ! i.atEnd ( ) ; i.advance ( ) ) {
      sb.append ( i.get ( ) + " " ) ;
    }
    assertEquals ( sb.toString ( ) , s ) ;
  }
  /**
   */
  private void fillOut ( final BinaryTree<String> bt ) {
    bt.add ( "Alpha" ) ;
    bt.add ( "Beta" ) ;
    bt.add ( "Delta" ) ;
    bt.add ( "Eta" ) ;
    bt.add ( "alpha" ) ;
    bt.add ( "beta" ) ;
    bt.add ( "delta" ) ;
    bt.add ( "eta" ) ;
  }
}
