/*
 *  ADS – A Java package of algorithms and data structures for educational purposes.  This software was
 *  originally created for the book Winder & Roberts, "Developing Java Software", Wiley.
 *
 *  Copyright © 1997–2008,2010 Russel Winder
 *
 *  This file is part of ADS.
 *
 *  ADS is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General
 *  Public License as published by the Free Software Foundation, either version 3 of the License, or (at
 *  your option) any later version.
 *
 *  ADS is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
 *  implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General
 *  Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License along with ADS.  If not, see
 *  <http://www.gnu.org/licenses/>.
 */
package org.devjavasoft.ads.tests.filesort ;

import org.devjavasoft.ads.Comparator ;
import org.devjavasoft.ads.sorting.SortableRecord ;
import org.devjavasoft.ads.sorting.SortableRecordInformation ;
import org.devjavasoft.ads.sorting.SortableRecordReader ;
import org.devjavasoft.ads.sorting.SortableRecordWriter ;

import java.io.File ;
import java.io.IOException ;

/**
 *  A Factory of the objects needed by file sorting methods.  Use Prototype.
 *
 *  @author Russel Winder
 */
public class TestRecordInformation implements SortableRecordInformation {
  private final Comparator<SortableRecord> c ;
  private final Comparator<SortableRecord> e ;
  public TestRecordInformation ( final Comparator<SortableRecord> c  , final Comparator<SortableRecord> e ) {
    this.c = c ;
    this.e = e ;
  }
  @Override public Comparator<SortableRecord> getComparator ( ) { return c ; }
  @Override public Comparator<SortableRecord> getEqualToComparator ( ) { return e ; }
  @Override public SortableRecordReader getSortableRecordReader ( final File f ) throws IOException {
    return new TestRecordReader ( f ) ;
  }
  @Override public SortableRecordWriter getSortableRecordWriter ( final File f ) throws IOException {
    return new TestRecordWriter ( f ) ;
  }
}
