/*
 *  ADS – A Java package of algorithms and data structures for educational purposes.  This software was
 *  originally created for the book Winder & Roberts, "Developing Java Software", Wiley.
 *
 *  Copyright © 1997–2008,2010 Russel Winder
 *
 *  This file is part of ADS.
 *
 *  ADS is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General
 *  Public License as published by the Free Software Foundation, either version 3 of the License, or (at
 *  your option) any later version.
 *
 *  ADS is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
 *  implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General
 *  Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License along with ADS.  If not, see
 *  <http://www.gnu.org/licenses/>.
 */
package org.devjavasoft.ads.tests.filesort ;

import org.devjavasoft.ads.sorting.SortableRecord ;
import org.devjavasoft.ads.sorting.SortableRecordReader ;

import java.io.BufferedReader ;
import java.io.File ;
import java.io.FileReader ;
import java.io.IOException ;

/**
 *  A reader of test records.
 *
 *  @author Russel Winder
 */
public class TestRecordReader implements SortableRecordReader {
  private final BufferedReader reader ;
  public TestRecordReader ( final File file ) throws IOException {
    reader = new BufferedReader ( new FileReader ( file ) ) ;
  }
  public TestRecordReader ( final String fileName ) throws IOException {
    reader = new BufferedReader ( new FileReader ( fileName ) ) ;
  }
  @Override public SortableRecord readSortableRecord ( ) throws IOException {
    final String s = reader.readLine ( ) ;
    return ( s == null ) ? null : new TestRecord ( s ) ;
  }
  @Override public void close ( ) throws IOException { reader.close ( ) ; }
  @Override protected void finalize ( ) throws IOException , Throwable { super.finalize ( ) ; close ( ) ; }
  @Override public void mark ( final int lookAheadLimit ) throws IOException { reader.mark ( lookAheadLimit ) ; }
  @Override public void reset ( ) throws IOException { reader.reset ( ) ; }
}
